#!/bin/sh

# . /lib/security-functions.sh

clear

if [ 0 -eq "$(kysec_is_enabled)" ]; then
	return 0
fi

kysec_tmp=/etc/kysec/tmp
kysec_whlist_db=/etc/kysec/db/whlist.db
kysec_init=/usr/sbin/kysec-init

relabel_system_for_whlist()
{
	show_message "正在扫描文件系统..."
	[ ! -d ${NEWROOT}/$kysec_tmp ] && mkdir -p -m 0777 ${NEWROOT}/$kysec_tmp
	rm -f ${NEWROOT}/$kysec_tmp/*
	find ${NEWROOT} -depth -maxdepth 1 \( -path ${NEWROOT}/proc -o -path ${NEWROOT}/run -o -path ${NEWROOT}/sys -o -path ${NEWROOT}/dev -o -path ${NEWROOT}/tmp -o -path ${NEWROOT}/cdrom -o -path ${NEWROOT}/media -o -path ${NEWROOT}/mnt -o -path ${NEWROOT}/box -o -path ${NEWROOT}/backup -o -path ${NEWROOT}/lost+found -o -path ${NEWROOT} \) -prune -o -print 1>${NEWROOT}/$kysec_tmp/scanf
	cat ${NEWROOT}/$kysec_tmp/scanf | xargs -i -P $(nproc) find {} \( -type f -o -type l \) -print 1>>${NEWROOT}/$kysec_tmp/scan
	new=$(echo ${NEWROOT} | sed 's#\/#\\\/#g')
	sed -i 's/^'${new}'//g' ${NEWROOT}/$kysec_tmp/scan
	split -l 3333 ${NEWROOT}/$kysec_tmp/scan -d -a 4 scan_
	mv scan_* ${NEWROOT}/$kysec_tmp

	show_message "正在初始化系统执行控制安全标记，请稍候..."
	cat ${NEWROOT}/$kysec_tmp/scanf | xargs -i -P $(nproc) ${kysec_init} --set-xattr {}
	${kysec_init} --rootmnt ${NEWROOT} --set-special

	setfattr -n security.kysec -v none:none:trusted ${NEWROOT}
}

relabel_system_from_whlist()
{
	show_message "正在初始化系统执行控制安全标记，请稍候..."
	${kysec_init} --rootmnt ${NEWROOT} --from-whlist
}

relabel_file=${NEWROOT}/.exectl

export exectl=
export relabel_flag=

get_relabel_flag()
{
	for x in $(cat /proc/cmdline); do
		case $x in
		exectl=*)
			exectl=${x#exectl=}
			;;
		esac
	done

	if [ -f "$relabel_file" -o "x${exectl}" = "x1" ]; then
		relabel_flag=$(sqlite3 "${NEWROOT}/$kysec_whlist_db" "select * from relabel_status;")
       	fi
	
	#if [ "x$relabel_flag" = "x0" ]; then
	#	sqlite3 "${NEWROOT}/$kysec_whlist_db" "update relabel_status set relabel=1;"
	#fi	
}

do_remount()
{
	old_value=`cat $kysec_status_path`
	mount -o remount,rw ${root_device} ${NEWROOT}
	set_kysec_status 4

	mount -t proc -o nodev,noexec,nosuid proc "${NEWROOT}/proc"
	mount -t sysfs -o nodev,noexec,nosuid sys "${NEWROOT}/sys"
	mount -t devtmpfs udev "${NEWROOT}/dev"

    if [ -f ${NEWROOT}/usr/sbin/lvm -a ${is_lvm} -eq 1 ];then
        chroot "${NEWROOT}" /usr/sbin/lvm vgchange -ay
    fi

	chroot "${NEWROOT}" /bin/mount -a

	if [ "$1" = "all" ]; then
		relabel_system_for_whlist
	elif [ "$1" = "whlist" ]; then
		relabel_system_from_whlist
	fi

	rm -f $relabel_file
	set_kysec_status $old_value

	umount "${NEWROOT}/dev"
	umount "${NEWROOT}/sys"
	umount "${NEWROOT}/proc"
}

export exectl=
export is_lvm=0

for x in $(cat /proc/cmdline); do
	case $x in
	exectl=*)
		exectl=${x#exectl=}
		;;
    rd.lvm.lv=*)
        is_lvm=1
        ;;
	esac
done

get_relabel_flag

if [ -n "$relabel_flag" ];then
	# disable kernel messages temporarily
	level=`cat /proc/sys/kernel/printk | awk '{print $1}'`
	show_message "文件系统Kysec安全标记需要重新初始化"
	echo 1 > /proc/sys/kernel/printk
	if [ "x$relabel_flag" = "x0" ]; then
		do_remount all
	elif [ "x$relabel_flag" = "x1" ]; then
		do_remount whlist
	fi
	# enable kernel messages
	echo "$level" > /proc/sys/kernel/printk
	clear_message
fi

clear
